#!/usr/bin/perl
# ----------------------------------------------------------------------------
#
# komazou.pl --- Photozou Uploader Command Line Version
#
# This program uploads pictures from command line to
# Photozou(http://photozou.jp), a picture Social Networking Service.
# Photozou is a trademark of Unoh Inc.(http://www.unoh.net)
#
# Copyright (c) 2005 Junya Ishihara <jishiha@users.sourceforge.jp>
#
#   This file is part of Komazou
#
#   Komazou is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   Komazou is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with Komazou; if not, write to the Free Software
#   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
#
#   Modified by    Junya Ishihara <jishiha@users.sourceforge.jp>
#
#   Special thanks to:
#   - Hiroshi Yuki (The original code of this program is very much based on
#  Hatena Diary Writer(http://www.hyuki.com/techinfo/hatena_diary_writer.html))
#
# ----------------------------------------------------------------------------

use strict;
my $VERSION = "0.0.1";

use LWP::UserAgent;
use HTTP::Request::Common;
use HTTP::Cookies;
use File::Basename;
use Getopt::Std;

# Prototypes.
sub login();
sub logout();
sub upload($$$$$);
sub print_debug(@);
sub print_message(@);
sub error_exit(@);
sub load_config();

# Photozou mail address (if empty, I will ask you later).
my $mail_address = '';
# Photozou password (if empty, I will ask you later).
my $password = '';

# Default file names.
my $cookie_file = 'cookie.txt';
my $config_file = 'config.txt';

# Photozou URL.
my $photozou_url = 'http://photozou.jp';
my $photozou_api_url = 'http://photozou.jp/photo_uploader.php';

# Option for LWP::UserAgent.
my %ua_option = (
    agent => "Komazou/$VERSION",
    timeout => 180,
);

# Other variables.
my $cookie_jar;
my $user_agent;
my $dummyinput = '';
my $title;

my ($day, $month, $year) = (localtime)[3,4,5];

# Handle command-line option.
my %cmd_opt = (
    'd' => 0,   # "debug" flag.
    'm' => "",  # "mail_address" option.
    'p' => "",  # "password" option.
    'a' => "",  # "agent" option.
    'T' => "",  # "timeout" option.
    'c' => 0,   # "cookie" flag.
    'n' => "",  # "config file" option.
);

$Getopt::Std::STANDARD_HELP_VERSION = 1;
getopts("dm:p:a:T:cn:", \%cmd_opt) or error_exit("Unknown option.");

if ($cmd_opt{d}) {
    print_debug("Debug flag on.");
    print_debug("Cookie flag on.") if $cmd_opt{c};
    &VERSION_MESSAGE();
}

# Override config file name (before load_config).
$config_file = $cmd_opt{n} if $cmd_opt{n};

# Override global vars with config file.
load_config() if -e($config_file);

# Override global vars with command-line options.
$mail_address = $cmd_opt{m} if $cmd_opt{m};
$password = $cmd_opt{p} if $cmd_opt{p};
$ua_option{agent} = $cmd_opt{a} if $cmd_opt{a};
$ua_option{timeout} = $cmd_opt{T} if $cmd_opt{T};

# Start.
&main;

# no-error exit.
exit(0);

# Main sequence.
sub main {
    my $count = 0;
    my @files = @ARGV;
    my ($basename, $dir, $ext);

    # Login if necessary.
    login() unless ($user_agent);

    # Process it.
    my $file;
    foreach $file (@files) {

        # Parse file name.
        ($basename, $dir, $ext) = fileparse($file, '\..*');
        print_debug("main: file parsing. \$basename: $basename, \$dir: $dir, \$ext: $ext.");

        # Check file name.
#        next unless ($file =~ /.+\.jpg$/);

        # Check if it is a file.
        next unless (-f $file);

        sleep(1);

        print "Title \[$basename\]: ";
        chomp($title = <STDIN>);
        $title = $basename if ($title eq '');

        print_message("Upload $file.");

        print_debug("main: upload($file, $title, $year, $month, $day)");
        upload($file, $title, $year, $month, $day);
        print_message("Upload OK.");

        $count++;
    }

    # Logout if necessary.
    logout() if ($user_agent);

    print_message("No files are posted.") if ($count == 0);
}

# Login using photozou simple API.
sub login() {
    my $login_url = $photozou_api_url;

    $user_agent = LWP::UserAgent->new(%ua_option);

    # Ask username if not set.
    unless ($mail_address) {
        print "Mail Address: ";
        chomp($mail_address = <STDIN>);
    }

    # Ask password if not set.
    unless ($password) {
        print "Password: ";
        chomp($password = <STDIN>);
    }

    my %form;

    $form{mode} = 'login';
    $form{email} = $mail_address;
    $form{password} = $password;

    my $r; # Response.

    print_message("Login to $login_url as $form{email}.");

    $r = $user_agent->simple_request(
        HTTP::Request::Common::POST("$login_url", \%form, 'Content_Type' => 'multipart/form-data')
    );

    print_debug("login: " . $r->status_line);

    print_debug("login: \$r = " . $r->content());

    unless ($r->content() =~ /0 Succeeded to login/) {
        error_exit("login: Unexpected response: ", $r->status_line);
    }

    print_message("Login OK.");

    print_debug("login: Making cookie jar.");

    $cookie_jar = HTTP::Cookies->new;
    $cookie_jar->extract_cookies($r);
    $cookie_jar->save($cookie_file) if $cmd_opt{c};

    print_debug("login: \$cookie_jar = " . $cookie_jar->as_string);
}

# Logout using photozou simple API.
sub logout() {
    my $logout_url = $photozou_api_url;

    my %form;

    $form{mode} = 'logout';

    my $r; # Response.

    print_message("Logout from $photozou_url.");

    $r = $user_agent->simple_request(
        HTTP::Request::Common::POST("$logout_url", \%form, 'Content_Type' => 'multipart/form-data')
    );

    print_debug("logout: " . $r->status_line);

    print_debug("logout: \$r = " . $r->content());

    unless ($r->content() =~ /0 Succeeded to logout/) {
        error_exit("logout: Unexpected response: ", $r->status_line);
    }

    print_message("Logout OK.");
}

# Upload picture using photozou simple API.
sub upload($$$$$) {
    my ($file, $title, $year, $month, $day) = @_;

    print_debug("upload: $file.");

    $user_agent->cookie_jar($cookie_jar);

    my $uploadurl = $photozou_api_url;

    my $r = $user_agent->simple_request(
        HTTP::Request::Common::POST($uploadurl,
            Content_Type => 'multipart/form-data',
            Content => [
                dummyinput => $dummyinput,
                mode => 'add_photo',
                'photo_title' => $title,
                'photo' => [$file],
                year => $year + 1900,
                month => $month + 1,
                day => $day,
            ]
        )
    );

    print_debug("upload: ", $r->status_line);

    print_debug("upload: ", $r->content());

    unless ($r->content() =~ /0 Succeeded to regist/) {
        error_exit("upload: Unexpected response: ", $r->status_line);
    }

    print_debug("upload: returns 1 (OK).");
    return 1;
}

# Show version message. This is called by getopts.
sub VERSION_MESSAGE {
    print <<"EOD";
Komazou Version $VERSION
Copyright (C) 2005 Junya Ishihara

Komazou comes with ABSOLUTELY NO WARRANTY.
This is free software, and you are welcome to redistribute it
under certain conditions; see the file named LICENSE in the Komazou
ditribution for details.
EOD
}

# Debug print.
sub print_debug(@) {
    if ($cmd_opt{d}) {
        print "DEBUG: ", @_, "\n";
    }
}

# Print message.
sub print_message(@) {
    print @_, "\n";
}

# Error exit.
sub error_exit(@) {
    print "ERROR: ", @_, "\n";
    unlink($cookie_file);
    exit(1);
}

# Show help message. This is called by getopts.
sub HELP_MESSAGE {
    print <<"EOD";

Usage: perl $0 [Options]

Options:
    --version       Show version.
    --help          Show this message.
    -d              Debug. Use this switch for verbose log.
    -m mail_address Mail address. Specify mail address.
    -p password     Password. Specify password.
    -a agent        User agent. Default value is Komazou/$VERSION.
    -T seconds      Timeout. Default value is 180.
    -c              Cookie. Skip login/logout if $cookie_file exists.
    -n config_file  Config file. Default value is $config_file.

Config file example:
#
# $config_file
#
mail_address:yourmailaddress
password:yourpassword
cookie:cookie.txt
EOD
}

# Load config file.
sub load_config() {
    print_debug("Loading config file ($config_file).");
    if (not open(CONF, $config_file)) {
        error_exit("Can't open $config_file.");
    }
    while (<CONF>) {
        chomp;
        if (/^\#/) {
            # skip comment.
        } elsif (/^$/) {
            # skip blank line.
        } elsif (/^mail_address:([^:]+)$/) {
            $mail_address = $1;
            print_debug("load_config: mail_address:$mail_address");
        } elsif (/^password:(.*)$/) {
            $password = $1;
            print_debug("load_config: password:********");
        } elsif (/^cookie:(.*)$/) {
            $cookie_file = glob($1);
            $cmd_opt{c} = 1; # If cookie file is specified, Assume '-c' is given.
            print_debug("load_config: cookie:$cookie_file");
        } else {
            error_exit("Unknown command '$_' in $config_file.");
        }
    }
    close(CONF);
}
__END__
